' Fig. 15.9: SynchronizedBuffer.vb
' A synchronized shared buffer implementation.
Imports System.Threading

' this class represents a single shared int
Public Class SynchronizedBuffer
   Implements IBuffer
   ' buffer shared by producer and consumer threads
   Private bufferValue As Integer = -1

   ' occupiedBufferCount maintains count of occupied buffers
   Private occupiedBufferCount As Integer = 0

   ' property Buffer
   Property Buffer() As Integer Implements IBuffer.Buffer
      Get
         ' obtain lock on this object
         Monitor.Enter(Me)

         ' if there is no data to read, place invoking 
         ' thread in WaitSleepJoin state
         If occupiedBufferCount = 0 Then
            Console.WriteLine( _
               Thread.CurrentThread.Name & " tries to read.")
            DisplayState("Buffer empty. " & _
               Thread.CurrentThread.Name & " waits.")
            Monitor.Wait(Me) ' enter WaitSleepJoin state
         End If

         ' indicate that producer can store another value 
         ' because consumer is about to retrieve a buffer value
         occupiedBufferCount -= 1

         DisplayState(Thread.CurrentThread.Name & " reads " & bufferValue)

         ' tell waiting thread (if there is one) to 
         ' become ready to execute (Running state)
         Monitor.Pulse(Me)

         ' Get copy of buffer before releasing lock. 
         ' It is possible that the producer could be
         ' assigned the processor immediately after the
         ' monitor is released and before the return 
         ' statement executes. In this case, the producer 
         ' would assign a new value to buffer before the 
         ' return statement returns the value to the 
         ' consumer. Thus, the consumer would receive the 
         ' new value. Making a copy of buffer and 
         ' returning the copy ensures that the
         ' consumer receives the proper value.
         Dim bufferCopy As Integer = bufferValue

         ' release lock on this object
         Monitor.Exit(Me)

         Return bufferCopy
      End Get
      Set(ByVal value As Integer)
         ' acquire lock for this object
         Monitor.Enter(Me)

         ' if there are no empty locations, place invoking
         ' thread in WaitSleepJoin state
         If occupiedBufferCount = 1 Then
            Console.WriteLine( _
               Thread.CurrentThread.Name & " tries to write.")
            DisplayState("Buffer full. " & _
               Thread.CurrentThread.Name & " waits.")
            Monitor.Wait(Me) ' enter WaitSleepJoin state
         End If

         ' set new buffer value
         bufferValue = value

         ' indicate consumer can retrieve another value 
         ' because producer has just stored a buffer value
         occupiedBufferCount += 1

         DisplayState(Thread.CurrentThread.Name & _
            " writes " & bufferValue)

         ' tell waiting thread (if there is one) to 
         ' become ready to execute (Running state)
         Monitor.Pulse(Me)

         ' release lock on this object
         Monitor.Exit(Me)
      End Set
   End Property ' Buffer

   ' display current operation and buffer state
   Public Sub DisplayState(ByVal operation As String)
      Console.WriteLine("{0,-35}{1,-9}{2}" & vbCrLf, operation, _
         bufferValue, occupiedBufferCount)
   End Sub ' DisplayState
End Class ' SynchronizedBuffer

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************